@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  BOOSTED MULTIVARIATE TREES FOR LONGITUDINAL DATA (BOOSTMTREE)
-####  Version 1.3.0 (_PROJECT_BUILD_ID_)
+####  Version 1.4.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -41,10 +41,13 @@
 ####    email:  hemant.ishwaran@gmail.com
 ####    URL:    http://web.ccs.miami.edu/~hishwaran
 ####    --------------------------------------------------------------
-####    Amol Pande
-####    Division of Biostatistics
-####    1120 NW 14th Street
-####    University of Miami, Miami FL 33136
+####    Amol Pande, Ph.D.
+####    Assistant Staff,
+####    Thoracic and Cardiovascular Surgery
+####    Heart and Vascular Institute
+####    JJ4, Room 508B,
+####    9500 Euclid Ave,
+####    Cleveland Clinic, Cleveland, Ohio, 44195
 ####
 ####    email:  amoljpande@gmail.com
 ####    --------------------------------------------------------------
@@ -61,16 +64,23 @@
 ####**********************************************************************
 
 
-simLong <- function(n = 100,
+simLong <-  function(n = 100,
                      ntest = 0,
                      N = 5,
                      rho = 0.8,
                      type = c("corCompSym", "corAR1", "corSymm", "iid"),
                      model = c(0, 1, 2, 3),
+                     family = c("Continuous","Binary"),
                      phi = 1,
                      q = 0,
                      ...)
 {
+  if(length(family) != 1){
+    stop("Specify any one of the family")
+  }
+  if(any(is.na( match(family,c("Continuous","Binary"))))){
+    stop("family must be Continuous or Binary")
+  }
   type <- match.arg(type, c("corCompSym", "corAR1", "corSymm", "iid"))
   model <- as.numeric(match.arg(as.character(model), as.character(0:3)))
   dta <- data.frame(do.call("rbind", lapply(1:(n+ntest), function(i) {
@@ -106,16 +116,23 @@ simLong <- function(n = 100,
     }
     tm <- sample((1:(3 * N))/N, size = Ni, replace = TRUE)
     if (model == 0) {
-      y <- 1.5 + 2.5 * x1 - 1.2 * x3 - .6 * x4 + eps
+      l_pred <- 1.5 + 2.5 * x1 - 1.2 * x3 - .6 * x4 + eps
     }
     if (model == 1) {
-      y <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * x4 - .65 * tm  * x2   + eps
+      l_pred <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * x4 - .65 * tm  * x2   + eps
     }
     if (model == 2) {
-      y <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * x4 - .65 * (tm ^ 2) * (x2 ^ 2)   + eps
+      l_pred <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * x4 - .65 * (tm ^ 2) * (x2 ^ 2)   + eps
     }
     if (model == 3) {
-      y <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * exp(x4) - .65 * (tm ^ 2) * (x2 ^2) * x3  + eps
+      l_pred <- 1.5 + 2.5 * x1 - 1.2 * x3 - .2 * exp(x4) - .65 * (tm ^ 2) * (x2 ^2) * x3  + eps
+    }
+    mu <- GetMu(Linear_Predictor = l_pred,Family = family)
+    if(family == "Continuous"){
+      y <- mu
+    }
+    else {
+      y <- rbinom(n = Ni,size = 1,prob = mu)
     }
     cbind(matrix(x, nrow = Ni, ncol = length(x), byrow = TRUE),
           tm, rep(i, Ni), y)
@@ -124,16 +141,16 @@ simLong <- function(n = 100,
   colnames(dta) <- c(paste("x", 1:d, sep = ""), "time", "id", "y")
   dtaL <- list(features = dta[, 1:d], time = dta$time, id = dta$id, y = dta$y) 
   if (model == 0) {
-    f.true <- "y ~ x1 + x3 + x4"
+    f.true <- "y ~ g( x1 + x3 + x4 )"
   }
   if (model == 1) {
-    f.true <- "y ~ x1 + x3 + x4 + I(time * x2)"
+    f.true <- "y ~ g( x1 + x3 + x4 + I(time * x2) )"
   }
   if (model == 2) {
-    f.true <- "y ~ x1 + x3 + x4 + I(time^2 * x2^2)"
+    f.true <- "y ~ g( x1 + x3 + x4 + I(time^2 * x2^2) )"
   }
   if (model == 3) {
-    f.true <- "y ~ x1 + x3 + exp(x4) + I(time^2 * x2^2 * x3)"
+    f.true <- "y ~ g( x1 + x3 + exp(x4) + I(time^2 * x2^2 * x3) )"
   }
   trn <- c(1:sum(dta$id <= n))
   return(invisible(list(dtaL = dtaL, dta = dta, trn = trn, f.true = f.true)))
